package de.fhw.swtprojekt.hibernatetest;

import java.util.List;
import java.util.Set;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;

/**Testprogramm fr Hibernate: legt eine Runde Daten an und wertet sie aus.
 * 
 * @author Wolfgang Knauf
 *
 */
public class HibernateTestMain
{
	/**Main-Methode, hier passiert die gesamte Logik.
	 * @param args Kommandozeile, hier egal.
	 */
	public static void main(String[] args)
	{

		try
		{
			// Create the SessionFactory from hibernate.cfg.xml
			SessionFactory sessionFactory = new Configuration().configure().buildSessionFactory();

			Session session = sessionFactory.openSession();
			
			//Schritt 1: Anlegen von Kuchen und Zutaten:
			session.getTransaction().begin();

			Kuchen kuchen1 = new Kuchen ();
			kuchen1.setName("Ksekuchen");
			session.save(kuchen1);
			
			Kuchen kuchen2 = new Kuchen ();
      kuchen2.setName("Streuselkuchen");
      session.save(kuchen2);
      
			Zutat zutat1 = new Zutat ();
			zutat1.setName("Mehl");
      session.save(zutat1);
      
      Zutat zutat2 = new Zutat ();
      zutat2.setName("Milch");
      session.save(zutat2);
      
			session.getTransaction().commit();
			
			//Laden aller Kuchen:
			session.getTransaction().begin();
			@SuppressWarnings("unchecked")
		  List<Kuchen> listKuchen = session.getNamedQuery("alle_kuchen").list();
			for (Kuchen kuchen : listKuchen)
      {
        System.out.println("Geladener Kuchen: " + kuchen);
      }
			
			//Verknpfung machen!
			kuchen1.getZutaten().add(zutat1);
			//Umgekehrt muss auch sein, sonst speichert er nix!
			zutat1.getKuchen().add(kuchen1);
			
			//In den Ksekuchen kommt auch Mehl:
			kuchen1.getZutaten().add(zutat2);
      zutat2.getKuchen().add(kuchen1);
      
      //Den Kuchen speichern, das nimmt die Zutaten mit.
			session.save(kuchen1);
			
			session.getTransaction().commit();
      
			
			//Neu laden der Kuchen und ihrer Zutaten:
      session.getTransaction().begin();
      @SuppressWarnings("unchecked")
      List<Kuchen> listKuchenMitZutaten = session.getNamedQuery("alle_kuchen").list();
      for (Kuchen kuchen : listKuchenMitZutaten)
      {
        Set<Zutat> setZutaten = kuchen.getZutaten();

        System.out.println("Geladener Kuchen: " + kuchen + ", hat " + kuchen.getZutaten().size() + " Zutaten: ");
        for (Zutat zutatZuKuchen : setZutaten)
        {
          System.out.println("\t" + zutatZuKuchen.toString());
        }
      }
			
			//Dichtmachen der Verbindung: explizit "Shutdown" als Query ausfhren!
			session.createSQLQuery("SHUTDOWN").executeUpdate();
			session.close();
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}

}
